/***************************************************
Musiktrainer V 0.1
MP3 Player gesteuert über einen SBUS Empfänger
****************************************************/

#define RXD1 32
#define TXD1 25

#include "DFRobotDFPlayerMini.h"  //https://github.com/DFRobot/DFRobotDFPlayerMini
#include "sbus.h"  // https://github.com/bolderflight/sbus

bfs::SbusRx sbus_rx(&Serial2,16, 17,true);
bfs::SbusData data;
DFRobotDFPlayerMini myDFPlayer;

void printDetail(uint8_t type, int value);
int vol = 25;
int tracks = 0;
int track = 1;
int chan = 7; // Kanal -1
int datum = 0;
int olddatum = 0;
int status = 0;
bool debug = true;

void setup()
{
  delay(3000);
  Serial1.begin(9600, SERIAL_8N1, RXD1, TXD1);
  if (debug) {Serial.begin(115200);}
  sbus_rx.Begin();
  
  if (debug){Serial.println();
  Serial.println(F("DFRobot DFPlayer Mini Demo"));
  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));}

  if (!myDFPlayer.begin(Serial1)) {  //Use Serial1 to communicate with mp3.
    if (debug){Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));}
    while(true);
  }
  if (debug){Serial.println(F("DFPlayer Mini online."));}
  delay(1000);
  myDFPlayer.volume(vol);    //Set volume value. From 0 to 30
  delay(500);
  tracks=myDFPlayer.readFileCounts();
  if (debug){Serial.print(F("Tracks: "));
    Serial.println(tracks);}
}

void loop()
{
  static unsigned long timer = millis();
  int incomingByte = 0;

if (sbus_rx.Read()) {
    /* Grab the received data */
    data = sbus_rx.data();
    datum = data.ch[chan];
      if ((debug) && (olddatum!=datum)){Serial.print(datum);
      Serial.println("\t");}
    olddatum = datum;
}

switch (datum) {
  case 990 ... 995:
    if (status != 2) {myDFPlayer.pause();delay(500);status = myDFPlayer.readState();if(debug){Serial.println(status);}}
  break;
  case 720 ... 725:
    if (status != 1) {myDFPlayer.play(track);while (status != 1) {delay(500);status=myDFPlayer.readState();} if(debug){Serial.println(status);}}
  break;    
  case 1259 ... 1264:
    if (status != 1) {myDFPlayer.loop(track); while (status != 1) {delay(500);status=myDFPlayer.readState();} if(debug){Serial.println(status);}}
  break;    
  case 449 ... 454:
    if (track > 1){track--;} else {track=tracks;} if(debug){Serial.println(track);} delay(500); status=3;
  break;
  case 1530 ... 1535:
    if (track < tracks){track++;} else {track=1;} if(debug){Serial.println(track);} delay(500); status=3;
  break;    
  case 170 ... 175:
    if (vol > 0) {vol--; myDFPlayer.volume(vol);delay(500);if(debug){Serial.println(myDFPlayer.readVolume());}}
  break;    
  case 1808 ... 1814:
    if (vol < 30) {vol++; myDFPlayer.volume(vol);delay(500);if(debug){Serial.println(myDFPlayer.readVolume());}}
  break;    
}

  if (myDFPlayer.available()) {
    printDetail(myDFPlayer.readType(), myDFPlayer.read()); //Print the detail message from DFPlayer to handle different errors and states.
  }
}

void printDetail(uint8_t type, int value){
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}